@@ -38,7 +38,7 @@
 #include <nm-vpn-plugin-utils.h>
 #include <nm-vpn-password-dialog.h>
 
-#include "src/nm-openswan-service.h"
+#include <../src/nm-openswan-service.h>
 
 #define KEYRING_UUID_TAG "connection-uuid"
 #define KEYRING_SN_TAG "setting-name"
