@@ -88,6 +88,8 @@ typedef struct {
 
 	GPid pid;
 	guint watch_id;
+	guint retry_id;
+	guint retries;
 	ConnectStep connect_step;
 	NMConnection *connection;
 
@@ -367,6 +369,11 @@ connect_cleanup (NMOpenSwanPlugin *self)
 		priv->pid = 0;
 	}
 
+	if (priv->watch_id) {
+		g_source_remove (priv->watch_id);
+		priv->watch_id = 0;
+	}
+
 	if (priv->io_id) {
 		g_source_remove (priv->io_id);
 		priv->io_id = 0;
@@ -441,6 +448,23 @@ connect_failed (NMOpenSwanPlugin *self,
 	if (do_stop)
 		ipsec_stop (self, NULL);
 	nm_vpn_plugin_failure (NM_VPN_PLUGIN (self), reason);
+	nm_vpn_plugin_set_state (NM_VPN_PLUGIN (self), NM_VPN_SERVICE_STATE_STOPPED);
+}
+
+static gboolean
+retry_cb (gpointer user_data)
+{
+	NMOpenSwanPlugin *self = NM_OPENSWAN_PLUGIN (user_data);
+	NMOpenSwanPluginPrivate *priv = NM_OPENSWAN_PLUGIN_GET_PRIVATE (self);
+	GError *error = NULL;
+
+	priv->retry_id = 0;
+
+	if (!connect_step (self, &error))
+		connect_failed (self, TRUE, error, NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED);
+	g_clear_error (&error);
+
+	return FALSE;
 }
 
 static void
@@ -466,17 +490,25 @@ child_watch_cb (GPid pid, gint status, gpointer user_data)
 	if (WIFEXITED (status)) {
 		ret = WEXITSTATUS (status);
 		if (ret)
-			g_warning ("Spawn: child %d exited with error code %d", pid, ret);
+			g_message ("Spawn: child %d exited with error code %d", pid, ret);
 	} else
 		g_warning ("Spawn: child %d died unexpectedly", pid);
 
 	/* Reap child */
 	waitpid (pid, NULL, WNOHANG);
 
+	if (ret != 0 && priv->retries) {
+		priv->retries--;
+		g_message ("Spawn: %d more tries...", priv->retries);
+		priv->retry_id = g_timeout_add (100, retry_cb, self);
+		return;
+	}
+
 	if (ret == 0) {
 		/* Success; do the next connect step */
 		do_stop = TRUE;
 		priv->connect_step++;
+		priv->retries = 0;
 		if (!connect_step (self, &error))
 			ret = 1;
 	}
@@ -623,6 +655,8 @@ nm_openswan_config_write (gint fd,
 	write_config_option (fd, " salifetime=24h\n");
 	write_config_option (fd, " ikelifetime=24h\n");
 	write_config_option (fd, " keyingtries=1\n");
+	if (libreswan)
+		write_config_option (fd, " cisco-unity=yes\n");
 	write_config_option (fd, " auto=add");
 
 	/* openswan requires a terminating \n (otherwise it segfaults) while
@@ -745,7 +779,6 @@ spawn_pty (int *out_stdout,
 		g_error ("PTY spawn: cannot exec '%s'", (char *) argv->pdata[0]);
 		_exit (-1);
 	}
-	g_ptr_array_free (argv, TRUE);
 
 	/* Close child side's pipes */
 	close (stderr_pipe[1]);
@@ -758,8 +791,10 @@ spawn_pty (int *out_stdout,
 		g_set_error (error, NM_VPN_PLUGIN_ERROR, NM_VPN_PLUGIN_ERROR_LAUNCH_FAILED,
 		             "PTY spawn failed for '%s' (%d)",
 		             (char *) argv->pdata[0], child_pid);
+		g_ptr_array_free (argv, TRUE);
 		return FALSE;
 	}
+	g_ptr_array_free (argv, TRUE);
 
 	/*  Set pipes non-blocking, so we can read big buffers
 	 *  in the callback without having to use FIONREAD
@@ -1007,6 +1042,8 @@ connect_step (NMOpenSwanPlugin *self, GError **error)
 		return TRUE;
 
 	case CONNECT_STEP_WAIT_READY:
+		if (!priv->retries)
+			priv->retries = 30;
 		if (!do_spawn (&priv->pid, NULL, NULL, error, priv->ipsec_path, "auto", "--ready", NULL))
 			return FALSE;
 		priv->watch_id = g_child_watch_add (priv->pid, child_watch_cb, self);
